@@ -12,6 +12,11 @@
 #include <QtGui/QScrollArea>
 #include <QtGui/QMenu>
 #include <QtGui/QPixmap>
+#include <QtCore/QRegExp>
+#include <QtCore/QMap>
+#include <QtCore/QVector>
+#include <QtCore/QTimer>
+#include <QDebug>
 
 #include "lookup.h"
 
@@ -190,6 +195,8 @@ Lookup::Lookup(QWidget *parent)
     connect(lookupline, SIGNAL(textEdited(QString)), this, SLOT(entryTextChanged(QString)));
     connect(lookupline, SIGNAL(returnPressed()), this, SLOT(dolookup()));
 
+    QTimer::singleShot(200, this, SLOT(dolookup()));
+
     //
     // Create the vertical answer sheet
     //
@@ -225,8 +232,13 @@ Lookup::dolookup()
     struct timeval start, stop;
     u_char buf[4096];
     char printbuf[4096];
+    int columns = 4;
 
     m_answers->clear();
+    m_answers->setColumnCount(columns);
+    QStringList headers;
+    headers << QString("Name") << QString("Type") << QString("TTL") << QString("Data");
+    m_answers->setHorizontalHeaderLabels(headers);
 
     // perform the lookup
     gettimeofday(&start, NULL);
@@ -270,10 +282,10 @@ Lookup::dolookup()
 //         do_section(&handle, ns_s_ar, RES_PRF_ADD, file);
 
         QMap<ns_sect, QStandardItem *> sections;
-        sections[ns_s_qd] = new QStandardItem(QString("Question: %1").arg(qdcount));
-        sections[ns_s_an] = new QStandardItem(QString("Answers: %1").arg(ancount));
-        sections[ns_s_ns] = new QStandardItem(QString("Authority: %1").arg(nscount));
-        sections[ns_s_ar] = new QStandardItem(QString("Additional: %1").arg(arcount));
+        sections[ns_s_qd] = new QStandardItem(QString("Question"));
+        sections[ns_s_an] = new QStandardItem(QString("Answers"));
+        sections[ns_s_ns] = new QStandardItem(QString("Authority"));
+        sections[ns_s_ar] = new QStandardItem(QString("Additional"));
 
         QStandardItem *results = new QStandardItem("Results");
         results->appendRow(sections[ns_s_qd]);
@@ -282,8 +294,11 @@ Lookup::dolookup()
         results->appendRow(sections[ns_s_ar]);
         m_answers->appendRow(results);
 
+        QStandardItem *theRealAnswer = 0;
+
         for(QMap<ns_sect, QStandardItem *>::iterator iter = sections.begin();
             iter != sections.end(); iter++) {
+            QMap<QString,QStandardItem *> dataItems;
             rrnum = 0;
             while(1) {
                 if (ns_parserr(&handle, iter.key(), rrnum, &rr)) {
@@ -295,17 +310,50 @@ Lookup::dolookup()
                     // error
                     return;
                 }
-                iter.value()->appendRow(new QStandardItem(printbuf));
+
+                // Create the row to display
+                QList<QStandardItem *> newRow;
+                QString rrType = QString(p_type(ns_rr_type(rr)));
+                newRow.push_back(new QStandardItem(QString(ns_rr_name(rr))));
+                newRow.push_back(new QStandardItem(rrType));
+                newRow.push_back(new QStandardItem(QString().number(ns_rr_ttl(rr))));
+
+                // remove the leading data from a printed representation that we've already extracted
+                QString dataBuffer = QString(printbuf);
+                dataBuffer.replace(QRegExp("^[-\\.\\w]+\\s+\\w+\\s+IN\\s+\\w+\\s+"), QString(""));
+                newRow.push_back(new QStandardItem(dataBuffer));
+
+                if(! dataItems.contains(rrType)) {
+                    QList<QStandardItem *> parentRow;
+                    parentRow.push_back(new QStandardItem(QString(ns_rr_name(rr))));
+                    parentRow.push_back(new QStandardItem(QString(p_type(ns_rr_type(rr)))));
+                    iter.value()->appendRow(parentRow);
+                    iter.value()->setColumnCount(parentRow.count());
+                    dataItems[rrType] = parentRow[0];
+                }
+
+                dataItems[rrType]->appendRow(newRow);
+                dataItems[rrType]->setColumnCount(newRow.count());
+
+                if (iter.key() == ns_s_an && m_queryType == ns_rr_type(rr)) {
+                    // remember that this is the real answer so we can expand it later
+                    theRealAnswer = dataItems[rrType];
+                    qDebug() << " found the answer";
+                }
+
                 rrnum++;
             }
         }
 
+        //
+        // Set the security results into the display
+        //
         QStandardItem *security = new QStandardItem("Security");
         m_answers->appendRow(security);
         
         if (val_isvalidated(val_status)) {
             m_securityStatus = 
-                new QStandardItem("Validated");
+                new QStandardItem("Status: Validated");
             m_answers->setSecurityStatus(m_securityStatus,
                                          QDNSItemModel::validated);
             m_resultsIcon->setPixmap(m_validated);
@@ -314,7 +362,7 @@ Lookup::dolookup()
 #endif
         } else if (val_istrusted(val_status)) {
             m_securityStatus = 
-                new QStandardItem("Trusted");
+                new QStandardItem("Status: Trusted");
             m_answers->setSecurityStatus(m_securityStatus,
                                          QDNSItemModel::trusted);
             m_resultsIcon->setPixmap(m_trusted);
@@ -322,7 +370,7 @@ Lookup::dolookup()
             m_answerView->setStyleSheet("QTreeView { background-color: #ffff96; }");
 #endif
         } else {
-            m_securityStatus = new QStandardItem("Bogus");
+            m_securityStatus = new QStandardItem("Status: Bogus");
             m_answers->setSecurityStatus(m_securityStatus,
                                          QDNSItemModel::bad);
             m_resultsIcon->setPixmap(m_bad);
@@ -332,20 +380,27 @@ Lookup::dolookup()
         }
 
         security->appendRow(m_securityStatus);
-        security->appendRow(new QStandardItem(p_val_status(val_status)));
+        security->appendRow(new QStandardItem(QString("code: ") + QString(p_val_status(val_status))));
 
         m_answerView->setExpanded(results->index(), true);
         m_answerView->setExpanded(security->index(), true);
         m_answerView->setExpanded(sections[ns_s_an]->index(), true);
+        if (theRealAnswer) {
+            m_answerView->setExpanded(theRealAnswer->index(), true);
+            qDebug() << "Expanding" << theRealAnswer->index();
+        }
 
     }
 
     m_answers->appendRow(new QStandardItem(QString("Time: %1 msec").arg(calculate_time_diff(&start, &stop))));
 
-
-    m_answerView->setHeaderHidden(true);
+    //m_answerView->setHeaderHidden(true);
     m_answerView->setRootIsDecorated(false);
-    
+    m_answers->emitChanges();
+    for(int i = 0 ; i < columns; i++) {
+        m_answerView->resizeColumnToContents(i);
+    }
+
     vlayout->invalidate();
 
     freeaddrinfo(aitop);
