@@ -28,6 +28,7 @@ class Lookup : public QWidget
     void setQueryType(int type);
     void setTypeText(const QString &label);
     QSize sizeHint();
+    void entryTextChanged(const QString &newtext);
 
   private:
     QLineEdit          *lookupline;
@@ -39,10 +40,14 @@ class Lookup : public QWidget
     QDNSItemModel      *m_answers;
     QStandardItem      *m_securityStatus;
 
+    // Icons
+    QPixmap             m_validated, m_trusted, m_bad, m_unknown;
+
   public:
     static const int  fields = 4;
     QLabel           *labels[fields];
     QLabel           *values[fields];
+    QLabel            *m_resultsIcon;
     bool              found;
     QPushButton       *m_queryButton;
     int               m_queryType;
